package lib

import (
	"fmt"
	"os"
	"strings"

	. "gopkg.in/check.v1"
)

func (s *OssutilCommandSuite) TestMakeBucket(c *C) {
	bucketName := bucketNamePrefix + randLowStr(10)
	s.putBucket(bucketName, c)

	// put bucket already exists
	s.putBucket(bucketName, c)

	// get bucket stat
	bucketStat := s.getStat(bucketName, "", c)
	c.Assert(bucketStat[StatName], Equals, bucketName)
	c.Assert(bucketStat[StatACL], Equals, "private")

	s.removeBucket(bucketName, true, c)

	bucketName = bucketNamePrefix + randLowStr(10)
	showElapse, err := s.putBucketWithACL(bucketName, "public-read-write")
	c.Assert(err, IsNil)
	c.Assert(showElapse, Equals, true)

	bucketStat = s.getStat(bucketName, "", c)
	c.Assert(bucketStat[StatName], Equals, bucketName)
	c.Assert(bucketStat[StatACL], Equals, "public-read-write")

	s.removeBucket(bucketName, true, c)
}

func (s *OssutilCommandSuite) TestMakeBucketErrorName(c *C) {
	for _, bucketName := range []string{"中文测试", "a"} {
		command := "mb"
		args := []string{CloudURLToString(bucketName, "")}
		str := ""
		options := OptionMapType{
			"endpoint":        &str,
			"accessKeyId":     &str,
			"accessKeySecret": &str,
			"stsToken":        &str,
			"configFile":      &configFile,
		}
		showElapse, err := cm.RunCommand(command, args, options)
		c.Assert(err, NotNil)
		c.Assert(showElapse, Equals, false)

		showElapse, err = s.rawGetStat(bucketName, "")
		c.Assert(err, NotNil)
		c.Assert(showElapse, Equals, false)
	}
}

func (s *OssutilCommandSuite) TestMakeBucketErrorACL(c *C) {
	bucketName := bucketNamePrefix + randLowStr(10)
	for _, language := range []string{DefaultLanguage, EnglishLanguage, LEnglishLanguage, "unknown"} {
		for _, acl := range []string{"default", "def", "erracl"} {
			showElapse, err := s.rawPutBucketWithACLLanguage([]string{CloudURLToString(bucketName, "")}, acl, language)
			c.Assert(err, NotNil)
			c.Assert(showElapse, Equals, false)

			showElapse, err = s.rawGetStat(bucketName, "")
			c.Assert(err, NotNil)
			c.Assert(showElapse, Equals, false)
		}
	}
	s.removeBucket(bucketName, true, c)
}

func (s *OssutilCommandSuite) TestMakeBucketErrorOption(c *C) {
	bucketName := bucketNamePrefix + randLowStr(10)
	command := "mb"
	args := []string{CloudURLToString(bucketName, "")}
	str := ""
	ok := true
	options := OptionMapType{
		"endpoint":        &str,
		"accessKeyId":     &str,
		"accessKeySecret": &str,
		"stsToken":        &str,
		"configFile":      &configFile,
		"shortFormat":     &ok,
	}
	showElapse, err := cm.RunCommand(command, args, options)
	c.Assert(err, NotNil)
	c.Assert(showElapse, Equals, false)
}

func (s *OssutilCommandSuite) TestErrMakeBucket(c *C) {
	acl := "private"
	showElapse, err := s.rawPutBucketWithACL([]string{"os://"}, acl)
	c.Assert(err, NotNil)
	c.Assert(showElapse, Equals, false)

	showElapse, err = s.rawPutBucketWithACL([]string{CloudURLToString("", "")}, acl)
	c.Assert(err, NotNil)
	c.Assert(showElapse, Equals, false)
}

func (s *OssutilCommandSuite) TestMakeBucketIDKey(c *C) {
	bucketName := bucketNamePrefix + randLowStr(10)

	cfile := randStr(10)
	data := fmt.Sprintf("[Credentials]\nendpoint=%s\naccessKeyId=%s\naccessKeySecret=%s\n[Bucket-Endpoint]\n%s=%s", "abc", "def", "ghi", bucketName, "abc")
	s.createFile(cfile, data, c)

	command := "mb"
	str := ""
	args := []string{CloudURLToString(bucketName, "")}
	options := OptionMapType{
		"endpoint":        &str,
		"accessKeyId":     &str,
		"accessKeySecret": &str,
		"stsToken":        &str,
		"configFile":      &cfile,
	}
	showElapse, err := cm.RunCommand(command, args, options)
	c.Assert(err, NotNil)

	options = OptionMapType{
		"endpoint":        &endpoint,
		"accessKeyId":     &accessKeyId,
		"accessKeySecret": &accessKeySecret,
		"stsToken":        &str,
		"configFile":      &cfile,
	}
	showElapse, err = cm.RunCommand(command, args, options)
	c.Assert(err, IsNil)
	c.Assert(showElapse, Equals, true)

	os.Remove(cfile)

	s.removeBucket(bucketName, false, c)
}

func (s *OssutilCommandSuite) TestMakeBucketStorageClass(c *C) {
	bucketName := bucketNamePrefix + randLowStr(10)

	result := []string{StorageStandard, StorageIA, StorageArchive}
	for i, class := range []string{StorageStandard, StorageIA, StorageArchive, strings.ToUpper(StorageStandard), strings.ToLower(StorageIA), "arCHIvE"} {
		err := s.initPutBucketWithStorageClass([]string{CloudURLToString(bucketName, "")}, class)
		c.Assert(err, IsNil)
		err = makeBucketCommand.RunCommand()
		c.Assert(err, IsNil)

		bucketStat := s.getStat(bucketName, "", c)
		c.Assert(bucketStat[StatName], Equals, bucketName)
		c.Assert(bucketStat[StatStorageClass], Equals, result[i%3])

		// delete bucket
		s.removeBucket(bucketName, true, c)
	}

	// test error make bucket
	err := s.initPutBucketWithStorageClass([]string{CloudURLToString("", "")}, StorageStandard)
	c.Assert(err, IsNil)
	err = makeBucketCommand.RunCommand()
	c.Assert(err, NotNil)

	// test error make bucket
	err = s.initPutBucketWithStorageClass([]string{CloudURLToString("", "abc")}, StorageStandard)
	c.Assert(err, IsNil)
	err = makeBucketCommand.RunCommand()
	c.Assert(err, NotNil)

	// test error make bucket
	err = s.initPutBucketWithStorageClass([]string{CloudURLToString(bucketName, "abc")}, StorageStandard)
	c.Assert(err, IsNil)
	err = makeBucketCommand.RunCommand()
	c.Assert(err, NotNil)

	_, err = s.rawGetStat(bucketName, "")
	c.Assert(err, NotNil)
}

func (s *OssutilCommandSuite) TestMbCreateBucketWithRedundancy(c *C) {
	bucketName := bucketNamePrefix + randLowStr(10)
	command := "mb"
	args := []string{CloudURLToString(bucketName, "")}
	str := ""
	strRedundancy := "ZRS"
	options := OptionMapType{
		"endpoint":        &str,
		"accessKeyId":     &str,
		"accessKeySecret": &str,
		"stsToken":        &str,
		"configFile":      &configFile,
		"redundancyType":  &strRedundancy,
	}
	_, err := cm.RunCommand(command, args, options)
	c.Assert(err, IsNil)

	bucketStat := s.getStat(bucketName, "", c)
	fmt.Println(bucketStat)
	c.Assert(bucketStat[StatRedundancyType], Equals, strRedundancy)
	s.removeBucket(bucketName, true, c)

	// redundancyType is error
	strRedundancy = "LLL"
	_, err = cm.RunCommand(command, args, options)
	c.Assert(err, NotNil)

	// create without redundancyType
	delete(options, "redundancyType")
	_, err = cm.RunCommand(command, args, options)
	c.Assert(err, IsNil)
	bucketStat = s.getStat(bucketName, "", c)
	c.Assert(bucketStat[StatRedundancyType], Equals, "LRS")
	s.removeBucket(bucketName, true, c)
}
